;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el valor de una varible							;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro set variable, value
	ld a, value
	ld (variable), a
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el valor de una varible con el de otra				;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setEquals inVariable, setVariable
	ld a, (inVariable)
	ld (setVariable), a
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recupera el valor de una varible							;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro get variable
	ld a, (variable)
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Compara el valor de una varible con un valor				;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro condition type, variable, value, execute
	if "type" = "Equals"
		get variable
		cp value
		jp nc, execute
	elseif "type" = "NotEquals"
		get variable
		cp value
		ret c
	elseif "type" = "EqualsRet"
		get variable
		cp value
		ret nc
	elseif "type" = "EqualsCall"
		get variable
		cp value
		call nc, execute
	elseif "type" = "Zero"
		get variable
		cp value
		jp z, execute
	elseif "type" = "ZeroVar"
		get variable
		ld b, a
		get value
		cp b
		jp z, execute
	elseif "type" = "ZeroCall"
		get variable
		cp value
		call z, execute
	elseif "type" = "NotZero"
		get variable
		cp value
		jp nz, execute
	elseif "type" = "NotZeroVar"
		get variable
		ld b, a
		get value
		cp b
		jp nz, execute
	elseif "type" = "NotZeroCall"
		get variable
		cp value
		call nz, execute
	elseif "type" = "ZeroRet"
		get variable
		cp value
		ret z
	elseif "type" = "ZeroRetVar"
		get variable
		ld b, a
		get value
		cp b
		ret z
	elseif "type" = "MinorOrEquals"
		ld a, (value)
		ld b, a
		get variable
		cp b
		jp c, execute
	elseif "type" = "Major"
		ld a, (value)
		ld b, a
		get variable
		cp b
		jp nc, execute
	elseif "type" = "is"
		get variable
    	and value
    	cp value
    	jp z, execute
	elseif "type" = "isCall"
		get variable
    	and value
    	cp value
    	call z, execute
	elseif "type" = "isRet"
		get variable
    	and value
    	cp value
    	ret z
	elseif "type" = "isNot"
		get variable
		and value
		cp value
		jp nz, execute
	elseif "type" = "isNotCall"
		get variable
		and value
		cp value
		call nz, execute
	elseif "type" = "isNotRet"
		get variable
    	and value
    	cp value
    	ret nz
	endif
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Compara el valor de A con un valor y si se cumple ejecuta;;
;; la cunfión especificada									;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro ifEquals value, execute
	cp value
	jp nc, execute
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Decrementa el valor de una varible 						;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro decreases variable
	get variable
	dec a
	set variable, a
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Incrementa el valor de una varible 						;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro increases variable
	get variable
	inc a
	set variable, a
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza el recuadro de juego con el marcador           ;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderPanel valor
	ld hl, valor
	ld (punteroCuadro), hl
	call mapa_cuadro
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza el recuadro de juego con el marcador           ;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro cleanScreenArea valor, from_x, to_x, from_y, to_y, color
	set clean_from_x, from_x
	set clean_to_x, to_x
	set clean_from_y, from_y
	set clean_to_y, to_y
	set color_clean, color
	ld hl, valor
	ld (punteroCuadro), hl
	call area_clean
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza un tile en la pantalla de video	            ;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderTile graphic
	ld hl, graphic
	jp screen_graph
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Renderiza en las dos pantallas del doble buffer			;;
;; 															;;
;; Afecta: HL, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderScreen
	call renderSelectedScreen
	call DB_execute
	call renderSelectedScreen
	call DB_execute
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Incrementa la posición de memoria grafica para un tile	;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro nextTilePosition
	ld hl, (puntero_memoria)
	inc hl
	inc hl
	ld (puntero_memoria), hl
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Recupera en siguiente tile a renderizar					;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro nextRenderTile
	ld hl, (puntero_elemento)
	inc hl
	ld (puntero_elemento), hl
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Repite el mismo tile que se ha renderizado				;;
;; 															;;
;; Afecta: HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro repeatRenderTile
	ld hl, (puntero_elemento)
	dec hl
	ld (puntero_elemento), hl
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Evalúa si está activada la función de Repeat				;;
;; 															;;
;; Afecta: HL, A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro isRepeatFunction
	ld hl, (puntero_elemento)
	ld a, -#FF
	add a, (hl)
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el flag de inicialización de pantalla				;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setInitScreen state
	set initScreen, state
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el flag de reseteo									;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setReseteo state
	set reseteo, state
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Muestra un texto por pantalla							;;
;; 															;;
;; Afecta: HL, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro printString posX, posY, string
	ld a, posX
	ld (txt_pos_x), a
    ld (txt_pos_x_init), a
    ld a, posY
	ld (txt_pos_y), a
	ld hl, string
	call txt_pantalla
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Muestra un texto por pantalla							;;
;; 															;;
;; Afecta: HL, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro printNumber posX, posY, string
	ld a, posX
	ld (txt_pos_x), a
    ld (txt_pos_x_init), a
    ld a, posY
	ld (txt_pos_y), a
	ld hl, string
	call num_pantalla
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Muestra un spryte por pantalla con máscara				;;
;; 															;;
;; Afecta: DE, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderSpryte posX, posY, spryte
	ld a, posY
	ld (GR_pos_y), a
	ld a, posX
	ld (GR_pos_x), a
	call posiciona_en_pantalla
	ld de, spryte
	call fill_graph
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Muestra un personaje por pantalla con máscara			;;
;; 															;;
;; Afecta: DE, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderSubject posX, posY, spryte
	ld a, (posY)
	ld (GR_pos_y), a
	ld a, (posX)
	ld (GR_pos_x), a
	call posiciona_en_pantalla
	ld de, spryte
	call fill_graph
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Muestra un spryte por pantalla sin máscara				;;
;; 															;;
;; Afecta: DE, A, B											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro renderSpryteSimple posX, posY, spryte
	ld a, posY
	ld b, posX
	ld de, spryte
	call fill_graphic_simple
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el color de tinta									;;
;; 															;;
;; Afecta: A, B, C											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setInk ink, color1, color2
	ld b, color1				;; Color 1
	ld a, ink			    	;; Tinta seleccionada
    ld c, color2 			    ;; Color 2
    call &BC32					;; Llamar a SCR SET INK del firmware para cambiar el color
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el mode 											;;
;; 															;;
;; Afecta: A												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setMode mode
	ld a, mode			    	;;
    call &BC0E					;; Mode
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Limpiar la pantalla										;;
;; 															;;
;; Afecta: 													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro clearScreen
	call &BC14					;; Limpia la pantalla y el cursor
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Setea el color del borde de la pantalla					;;
;; 															;;
;; Afecta: 													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setBorder color1, color2
	ld b, color1				;; Color 1
    ld c, color2				;; Color 2    	
    call &BC38					;; Color borde 
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; VSync sincronizacion con el render						;;
;; 															;;
;; Afecta: B, C, A											;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setVSync iter
	ld  b, #F5 				;;[2] B = F5h ==> B has the address of PPI Port B, where we get information from VSYNC
	ld c, b
	wvs_wait{iter}:			;; [4] A = Status register from PPI port B
   	in  a, (c)         		;; [1] Move bit 0 of A to Carry (bit 0 contains VSYNC status)
   	rra               		;; [2/3] No Carry means No VSYNC, so loop While No Carry
   	jr nc, wvs_wait{iter}  	;; [3] Carry Set, VSYNC Active, Return
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Inicializa VSysc para el doble buffer					;;
;; 															;;
;; Afecta:													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro startVSync
	setVSync 1
	halt
	halt
	setVSync 2
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Espablece la memria donde se inicia la pila				;;
;; 															;;
;; Afecta:													;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro setStartStack mem_stack
	ld sp, mem_stack				;;Cambiamos la ubicación de la pila
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Deshabilita el firmware									;;
;; 															;;
;; Afecta:	HL												;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
macro disableFirmware 
	di
	im 1
	ld hl, #C9FB ; ei, ret
	ld (#0038),hl
	ei
endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; SCREENS																							;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

macro useScreen
	ld a, (activeScreen)
	ld b, a
    call select_screen
endm

macro delay time
	ld a, time
	call delayTimer
endm

macro insertStaticEntity posX, posY, type
	db ACTIVO, type							;; s_entity, s_type
	db posX, posY, posY, posX, posX			;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_RIGHT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000  
	db 10, 3								;; s_jump, s_jumping
	db MOV_HORIZONTAL, 2, 1					;; s_mov, s_ancho, s_alto
	db 30, 11								;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm

macro insertMoneda posX, posY
	db ACTIVO, OBJ_MONEDA					;; s_entity, s_type
	db posX, posY, posY, posX, posX + 1		;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_RIGHT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 10, 3								;; s_jump, s_jumping
	db MOV_HORIZONTAL, 1, 1					;; s_mov, s_ancho, s_alto
	db 30, 11								;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm

macro insertLlave posX, posY, posXTake, posYTake
	db ACTIVO, OBJ_LLAVE					;; s_entity, s_type
	db posX, posY, posY, posX, posX			;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_RIGHT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 10, 3								;; s_jump, s_jumping
	db MOV_HORIZONTAL, 1, 1					;; s_mov, s_ancho, s_alto
	db posXTake, posYTake					;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm

macro insertPuerta posX, posY
	db ACTIVO, OBJ_PUERTAABIERTA			;; s_entity, s_type
	db posX, posY, posY, posX, posX			;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_RIGHT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 10, 3								;; s_jump, s_jumping
	db MOV_HORIZONTAL, 1, 1					;; s_mov, s_ancho, s_alto
	db posX, posY							;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm

macro insertSpyder posX, posY, posYinicial, posYfinal
	db ACTIVO, ENEMY_SPIDER					;; s_entity, s_type
	db posX, posY, posY, posY, posYfinal	;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_UP, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 10, 3								;; s_jump, s_jumping
	db MOV_VERTICAL, 3, 2					;; s_mov, s_ancho, s_alto
	db 30, 11								;; s_takX, s_takY
	db posX, posYinicial					;; s_init_x, s_init_y
endm

macro insertHuevo posX, posY, takeX, takeY
	db ACTIVO, OBJ_HUEVO					;; s_entity, s_type
	db posX, posY, posY, posX, posX			;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_LEFT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 0, 3									;; s_jump, s_jumping
	db MOV_HORIZONTAL, 1, 1					;; s_mov, s_ancho, s_alto
	db takeX, takeY							;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm

macro insertEnemy posX, posY, posXStart, posXEnd, type
	db ACTIVO, type							;; s_entity, s_type
	db posX, posY, posY, posXStart, posXEnd	;; s_X, s_Y, s_base, s_limL, s_limR
	db ORIENTATION_LEFT, RENDER_STATE1		;; s_orient, s_status
	db 1, 1, 1, 1							;; s_posXC000, s_posYC000, s_posX8000, s_posY8000 
	db 10, 3								;; s_jump, s_jumping
	db MOV_HORIZONTAL, 2, 2					;; s_mov, s_ancho, s_alto
	db 30, 11								;; s_takX, s_takY
	db posX, posY							;; s_init_x, s_init_y
endm